@@ -21,20 +21,31 @@ package Bio::DB::HTS::Tabix;
 
 use Bio::DB::HTS;
 use Bio::DB::HTS::Tabix::Iterator;
-$Bio::DB::HTS::Tabix::VERSION = '2.5';
+$Bio::DB::HTS::Tabix::VERSION = '2.6';
 use strict;
 use warnings;
 
 use Scalar::Util qw/reftype/;
+use File::Spec;
+use Cwd;
 
 sub new {
   my $class         = shift;
   my (%args) = @_;
   my $filename = $args{filename};
   my $warnings = $args{warnings};
+  my $use_tmp_dir = $args{use_tmp_dir};
 
   # filename checks
   die "Tabix region lookup requires a gzipped, tabixed bedfile" unless $filename =~ /gz$/;
+  my $pdir    = getcwd;
+  my $tmpdir = File::Spec->tmpdir;
+
+  #change to the tmp dir to allow the C library to download remote index files
+  if( Bio::DB::HTS->is_remote($filename) && ( !defined $use_tmp_dir || $use_tmp_dir != 0) )
+  {
+    chdir($tmpdir);
+  }
 
   my $htsfile = Bio::DB::HTSfile->open($filename);
   my $tabix_index = tbx_open($filename);
@@ -44,6 +55,7 @@ sub new {
   {
     $header = join "\n", @{ $header } ;
   }
+  chdir($pdir);
 
   my $self = bless {
                     _htsfile => $htsfile,
@@ -55,6 +67,29 @@ sub new {
   return $self;
 }
 
+sub query_full {
+  my ($self, $chr, $start, $end) = @_;
+  my $region = $chr;
+  $region .= ':'; #trailing ':' required for odd chr containing ':'
+  if(defined $start) {
+    $region .= $start;
+    $region .= '-'.$end if(defined $end);
+  }
+  my $iter = tbx_query( $self->{_tabix_index}, $region );
+  unless ( $iter ) {
+    #this likely means the chromosome wasn't found in the tabix index, or it couldn't parse the provided region.
+    my $seqnames_hash = { map { $_ => 1 } @{ $self->seqnames } };
+    if ( not exists $seqnames_hash->{ $chr } ) {
+      #$self->log->warn("Specified chromosome '$chr' does not exist in file " . $self->_filename);
+    }
+    else {
+      die "Unable to get iterator for region '$region' in file ". $self->{_filename} . " -- htslib couldn't parse your region string";
+      }
+  }
+
+  return Bio::DB::HTS::Tabix::Iterator->new( _tabix_iter => $iter, _htsfile => $self->{_htsfile}, _tabix_index => $self->{_tabix_index} );
+}
+
 sub query {
     my ( $self, $region ) = @_;
 
@@ -73,21 +108,7 @@ sub query {
         }
     }
 
-    my $iter = tbx_query( $self->{_tabix_index}, $region );
-
-    unless ( $iter ) {
-      #this likely means the chromosome wasn't found in the tabix index, or it couldn't parse the provided region.
-      my $seqnames_hash = { map { $_ => 1 } @{ $self->seqnames } };
-      if ( not exists $seqnames_hash->{ $chr } ) {
-        #$self->log->warn("Specified chromosome '$chr' does not exist in file " . $self->_filename);
-      }
-      else {
-        die "Unable to get iterator for region '$region' in file ". $self->{_filename} . " -- htslib couldn't parse your region string";
-        }
-
-    }
-
-    return Bio::DB::HTS::Tabix::Iterator->new( _tabix_iter => $iter, _htsfile => $self->{_htsfile}, _tabix_index => $self->{_tabix_index} );
+    return $self->query_full($chr, $start, $end);
 }
 
 sub seqnames {
@@ -102,8 +123,12 @@ sub header {
 
 sub header_array {
     my $self = shift;
-    my @lines = split(/\n/,$self->{_header});
-    return @lines ;
+    if( $self->{_header} )
+    {
+      my @lines = split(/\n/,$self->{_header});
+      return @lines ;
+    }
+    return ;
 }
 
 
@@ -180,11 +205,14 @@ Returns all the header lines as a single scalar from the tabixed file
 
 Takes a single region like: '1:4000005-4000009' or '12:5000000'. The coordinate format is 0 or 1-based for start and stop positions depending on how the Tabix index file was created - by default this is 1.
 
+The index for remote files will be downloaded to the system temporary directory by default. Set use_tmp_dir to 0 to download to the current working directory instead.
+
+
 Here are some examples showing Tabix.
 
     use Bio::DB::HTS::Tabix;
 
-    my $tabix = Bio::DB::HTS::Tabix->new(filename => $file);
+    my $tabix = Bio::DB::HTS::Tabix->new(filename => $file, use_tmp_dir => 1);
 
     # Calling region 1
 
